/* 
 * tcl2array.c V1.01 - convert tk/tcl code to a C-code array of strings
 * 
 * Copyright (c) 1993, 1994 University of Southern California.
 * All rights reserved.                                            
 *                                                                
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation, advertising
 * materials, and other materials related to such distribution and use
 * acknowledge that the software was developed by the University of
 * Southern California, Information Sciences Institute.  The name of the
 * University may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 J. Touch / touch@isi.edu
 10/7/93
    
 modifications and version information appear at the end of this
 header comment.

 translate tcl code to a string array, i.e., char *[]
 the last element is the null string ""
 
 patterned loosly on tcl2c of nv (see notice below)
 
 tcl2array stringname < infile.tcl > outfile.c
 
 outfile is a C code file of a char *stringname[] 
 and a 
     int stringname_chars = n
     int stringname_lines = l
 where n is the number of characters in the array, overall.
 and l is the number of lines in the array.
 The last element of the array is "", so the array can be traversed
 as is, or with the size as a limit on the index.
 This information is used by embed_tcl.c, a library to use the resulting 
 stringname as the tcl source.
 
 This code is based on tcl2c.c, part of the nv3.2 release code, 
 which carries the following comments:

	Netvideo version 3.2
	Written by Ron Frederick <frederick@parc.xerox.com>

	Simple hack to translate a Tcl/Tk init file into a C string constant

	* Copyright (c) Xerox Corporation 1992. All rights reserved.
	*  
	* License is granted to copy, to use, and to make and to use derivative
	* works for research and evaluation purposes, provided that Xerox is
	* acknowledged in all documentation pertaining to any such copy or derivative
	* work. Xerox grants no other licenses expressed or implied. The Xerox trade
	* name should not be used in any advertising without its written permission.
	*  
	* XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
	* MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
	* FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
	* express or implied warranty of any kind.
	*  
	* These notices must be retained in any copies of any part of this software.
*/

/* MODIFICATIONS
   Version 1.01 July 1994
   
   Changed
     while (!feof(stdin)) {
       gets(linein);
       ...
     }
   to
     gets(linein);
     while (!feof(stdin)) {
       ...
       gets(linein);
     }
   to avoid replicating the last line of the TCL file.
   Thanks to Steve Olson at MIT for pointing this out.
       
*/

#include <stdio.h>

main(argc, argv)
     int argc;
     char *argv[];
{
  long charcount = 0;
  long linecount = 0;
  char *c, linein[1000];

  if (argc < 2) {
    fprintf(stderr, "Use as: %s string_array_name\n", argv[0]);
    exit(1);
  }

  printf("char *%s[] = {\n", argv[1]);
  gets(linein);
  while (!feof(stdin)) {
    /* no interior nulls */
    if (linein[0] != NULL) {
      linecount++;
      printf("\"");
      for (c=linein; *c != NULL; c++) {
	charcount += 1;
	switch (*c) {
	case '\"':
	  printf("\\\"");
	  break;
	case '\\':
	  printf("\\\\");
	  break;
	default:
	  putchar(*c);
	}
      }
      printf("\",\n");
    }
    gets(linein);
  }
  printf("\"\"\n};\n");
  printf("int %s_chars = %ld;\n",argv[1],charcount);
  printf("int %s_lines = %ld;\n",argv[1],linecount);
  exit(0);
  /*NOTREACHED*/
}
